@@ -42,11 +42,22 @@ def _parse_index(idx: Any) -> Union['List', int, str]:
 
     return idx if type(idx) is int else str(idx)
 
+def _float_div(a: Any, b: Any) -> 'Float':
+    a, b = float(a), float(b)
+    if b != 0: return Float(a / b)
+    if a == 0: return Float(math.nan)
+    return Float(math.inf if a > 0 else -math.inf)
+
 class Float(float):
     def __eq__(self, other: Any) -> bool:
         if isinstance(other, str): return wrap(float(self) == float(other))
         return wrap(float(self) == other)
 
+    def __str__(self) -> str:
+        if math.isnan(self): return 'NaN'
+        if math.isinf(self): return 'Infinity' if self > 0 else '-Infinity'
+        return str(+self)
+
     def __bool__(self) -> bool:
         return self != 0 and not math.isnan(self)
 
@@ -77,10 +88,10 @@ class Float(float):
     def __truediv__(self, other: Any) -> Any:
         other = wrap(other)
         if isinstance(other, List): return other.__rtruediv__(self)
-        return _scalar_op(self, other, lambda a, b: a / b)
+        return _scalar_op(self, other, _float_div)
     def __rtruediv__(self, other: Any) -> Any:
         if not is_wrapped(other): return wrap(other) / self
-        return _scalar_op(other, self, lambda a, b: a / b)
+        return _scalar_op(other, self, _float_div)
 
     def __floordiv__(self, other: Any) -> Any:
         other = wrap(other)
@@ -102,6 +113,7 @@ class Float(float):
         return Float(-float(self))
     def __pos__(self) -> Union[int, float]:
         vf = float(self)
+        if not math.isfinite(vf): return vf
         vi = int(vf)
         return vi if vi == vf else vf
     def __abs__(self) -> 'Float':
@@ -191,6 +203,16 @@ class List(list):
     def __bool__(self) -> bool:
         return True
 
+    def __delitem__(self, idx: Any) -> None:
+        idx = _parse_index(idx)
+        t = type(idx)
+        if t is List: pass
+        elif t is str: del self.__str_keys[idx]
+        elif t is int:
+            if idx < 0: del self.__str_keys[str(idx)]
+            elif idx < len(self): list.__delitem__(self, idx)
+            else: pass
+        else: assert False
     def __getitem__(self, idx: Any) -> Any:
         idx = _parse_index(idx)
         t = type(idx)
@@ -264,9 +286,15 @@ class List(list):
 
     def random(self) -> Any:
         if len(self) == 0: return wrap('')
-        return random.choice(self)
+        return wrap(random.choice(self))
     def last(self) -> Any:
-        return self[len(self) - 1]
+        if len(self) == 0: return wrap('')
+        return wrap(list.__getitem__(self, -1))
+    def pop(self) -> Any:
+        if len(self) == 0: return wrap('')
+        res = wrap(list.__getitem__(self, -1))
+        list.__delitem__(self, -1)
+        return res
     def all_but_first(self) -> 'List':
         if len(self) == 0: return List()
         return wrap(list.__getitem__(self, slice(1, len(self))))
@@ -336,6 +364,8 @@ def srange(a: Any, b: Any) -> List:
 
 def sqrt(value: Any) -> Any:
     return _list_unary_op(wrap(value), lambda x: wrap(math.sqrt(+x)))
+def lnot(value: Any) -> Any:
+    return _list_unary_op(wrap(value), lambda x: not x)
 
 def sin(value: Any) -> Any:
     return _list_unary_op(wrap(value), lambda x: wrap(math.sin(+x * (math.pi / 180))))
@@ -506,6 +536,13 @@ if __name__ == '__main__':
     assert len(v) == 6 and v == ['', '', '', '', '', 'test 8'] and v[5] == 'test 8' and is_wrapped(v[5])
     v[2] = 'test 9'
     assert len(v) == 6 and v == ['', '', 'test 9', '', '', 'test 8']
+    assert len(v) == 6 and v[wrap(-1)] == 'test 7' and v[wrap(-5)] == 'test 6' and v['hello pony'] == 'test 5' and v == ['', '', 'test 9', '', '', 'test 8']
+    del v[-1]
+    assert len(v) == 6 and v[wrap(-1)] == '' and v[wrap(-5)] == 'test 6' and v['hello pony'] == 'test 5' and v == ['', '', 'test 9', '', '', 'test 8']
+    del v['hello pony']
+    assert len(v) == 6 and v[wrap(-1)] == '' and v[wrap(-5)] == 'test 6' and v['hello pony'] == '' and v == ['', '', 'test 9', '', '', 'test 8'] and v[5] == 'test 8'
+    del v[3]
+    assert len(v) == 5 and v[wrap(-1)] == '' and v[wrap(-5)] == 'test 6' and v['hello pony'] == '' and v == ['', '', 'test 9', '', 'test 8'] and v[4] == 'test 8' and v[5] == ''
 
     assert abs(sin(1) - 0.01745240643728351) < 1e-10 and abs(cos(1) - 0.9998476951563913) < 1e-10 and abs(tan(1) - 0.017455064928217585) < 1e-10
     assert abs(asin(0.75) - 48.590377890729144) < 1e-10 and abs(acos(0.75) - 41.40962210927086) < 1e-10 and abs(atan(0.75) - 36.86989764584402) < 1e-10
@@ -519,10 +556,40 @@ if __name__ == '__main__':
     vv = v[[[4,2],2,[[3,1]]]] ; assert vv == [['e', 'c'], 'c', [['d', 'b']]] and is_wrapped(vv)
 
     v = wrap([])
+    vv = v.random() ; assert vv == '' and is_wrapped(vv)
     vv = v.all_but_first() ; assert vv == [] and is_wrapped(vv) and vv is not v and not identical(vv, v)
     v = wrap([7])
+    vv = v.random() ; assert vv == 7 and is_wrapped(vv)
     vv = v.all_but_first() ; assert vv == [] and is_wrapped(vv) and vv is not wrap([]).all_but_first()
     v = wrap([3, 6, 3, 8, 7])
     vv = v.all_but_first() ; assert vv == [6, 3, 8, 7] and is_wrapped(vv) and v == [3,6,3,8,7]
+    xx = vv.pop() ; assert xx == 7 and is_wrapped(xx) and vv == [6,3,8] and vv.last() == 8 and vv.last() == 8 and is_wrapped(vv.last())
+    xx = vv.pop() ; assert xx == 8 and is_wrapped(xx) and vv == [6,3] and vv.last() == 3 and vv.last() == 3 and is_wrapped(vv.last())
+    xx = vv.pop() ; assert xx == 3 and is_wrapped(xx) and vv == [6] and vv.last() == 6 and vv.last() == 6 and is_wrapped(vv.last())
+    xx = vv.pop() ; assert xx == 6 and is_wrapped(xx) and vv == [] and vv.last() == '' and vv.last() == '' and is_wrapped(vv.last())
+    xx = vv.pop() ; assert xx == '' and is_wrapped(xx) and vv == [] and vv.last() == '' and vv.last() == '' and is_wrapped(vv.last())
+    xx = vv.pop() ; assert xx == '' and is_wrapped(xx) and vv == [] and vv.last() == '' and vv.last() == '' and is_wrapped(vv.last())
+    v = wrap([3, 6, 3, 8, 7])
+    assert v == [3,6,3,8,7] and len(v) == 5
+    v.clear()
+    assert v == [] and len(v) == 0
+
+    assert str(wrap(34)) == '34' and str(wrap(-34.5)) == '-34.5'
+
+    assert bool(wrap(True)) == True and bool(wrap(False)) == False
+    assert bool(wrap(5)) and bool(wrap(-2.3)) and not wrap(0) and not wrap(0.0) and not wrap(math.nan)
+
+    assert lnot(wrap(True)) == False and lnot(wrap(False)) == True
+    assert lnot('') == True and lnot('h') == False and lnot(0) and lnot(math.nan)
+    v = lnot(wrap(['hello', '', True, [0, 4, False, -1], '0'])) ; assert v == [False, True, False, [True, False, True, False], False] and is_wrapped(v)
+
+    assert str(wrap(1) / wrap(0)) == 'Infinity' and str(wrap('1') / wrap('0')) == 'Infinity'
+    assert str(wrap(-1) / wrap(0)) == '-Infinity' and str(wrap('-1') / wrap('0')) == '-Infinity'
+    assert str(wrap(0) / wrap(0)) == 'NaN' and str(wrap('0') / wrap('0')) == 'NaN'
+    assert str(wrap(-0) / wrap(0)) == 'NaN' and str(wrap('-0') / wrap('0')) == 'NaN'
+
+    v = +wrap('Infinity') ; assert v > 0 and math.isinf(v)
+    v = +wrap('-Infinity') ; assert v < 0 and math.isinf(v)
+    v = +wrap('NaN') ; assert math.isnan(v)
 
     print('passed all snap wrapper tests')
